import { useState } from 'react';
import type { TotpAccount } from '../services/db';
import { useTotp } from '../hooks/useTotp';
import { CountdownBar } from './CountdownBar';
import { CopyButton } from './CopyButton';
import { deleteAccount } from '../services/db';

interface TotpCardProps {
  account: TotpAccount;
  onDelete: (id: number) => void;
}

export function TotpCard({ account, onDelete }: TotpCardProps) {
  const { totp, remainingTime } = useTotp(account);
  const [isMenuOpen, setIsMenuOpen] = useState(false);

  const handleDelete = async () => {
    if (account.id && window.confirm(`Are you sure you want to delete "${account.name}"?`)) {
      try {
        await deleteAccount(account.id);
        onDelete(account.id);
      } catch (error) {
        console.error('Failed to delete account:', error);
        alert('Error deleting account. Please try again.');
      }
    }
  };

  const formattedTotp = totp.length === 6 
    ? `${totp.slice(0, 3)} ${totp.slice(3, 6)}`
    : totp;

  return (
    <div className="bg-white dark:bg-slate-900 rounded-xl p-5 shadow-fluent-md hover:shadow-fluent-lg transition-shadow duration-300 flex flex-col space-y-4">
      {/* Card Header */}
      <div className="flex items-center justify-between">
        <div>
          <p className="text-sm text-slate-500 dark:text-slate-400">{account.issuer || ' '}</p>
          <p className="text-lg font-semibold text-slate-800 dark:text-slate-200">{account.name}</p>
        </div>
        <div className="relative">
          <button 
            onClick={() => setIsMenuOpen(!isMenuOpen)}
            onBlur={() => setTimeout(() => setIsMenuOpen(false), 150)}
            className="text-slate-400 hover:text-slate-600 dark:hover:text-slate-200 transition-colors p-1 rounded-full"
          >
            <svg xmlns="http://www.w3.org/2000/svg" className="h-5 w-5" viewBox="0 0 20 20" fill="currentColor">
              <path d="M10 6a2 2 0 110-4 2 2 0 010 4zM10 12a2 2 0 110-4 2 2 0 010 4zM10 18a2 2 0 110-4 2 2 0 010 4z" />
            </svg>
          </button>
          {isMenuOpen && (
            <div className="absolute right-0 mt-2 w-48 bg-white dark:bg-slate-800 rounded-md shadow-lg z-10 border border-slate-200 dark:border-slate-700">
              <button
                onMouseDown={(e) => e.preventDefault()}
                onClick={handleDelete}
                className="w-full text-left px-4 py-2 text-sm text-red-600 dark:text-red-400 hover:bg-slate-100 dark:hover:bg-slate-700 flex items-center space-x-2"
              >
                <svg xmlns="http://www.w3.org/2000/svg" className="h-4 w-4" fill="none" viewBox="0 0 24 24" stroke="currentColor" strokeWidth={2}>
                  <path strokeLinecap="round" strokeLinejoin="round" d="M19 7l-.867 12.142A2 2 0 0116.138 21H7.862a2 2 0 01-1.995-1.858L5 7m5 4v6m4-6v6m1-10V4a1 1 0 00-1-1h-4a1 1 0 00-1 1v3M4 7h16" />
                </svg>
                <span>Delete</span>
              </button>
            </div>
          )}
        </div>
      </div>

      {/* TOTP Code Display */}
      <div className="text-center">
        <p className="text-4xl font-mono tracking-widest text-primary-600 dark:text-primary-400">
          {formattedTotp}
        </p>
      </div>

      {/* Countdown and Actions */}
      <div className="space-y-3 pt-2">
        <CountdownBar remainingTime={remainingTime} period={account.period} />
        <div className="flex items-center justify-end space-x-2">
          <span className="text-xs text-slate-500 dark:text-slate-400 font-mono w-8 text-center">
            {remainingTime}s
          </span>
          <CopyButton textToCopy={totp} />
        </div>
      </div>
    </div>
  );
}
