import { useRef } from 'react';
import * as OTPAuth from 'otpauth';
import { getAllAccounts, importAccounts, type TotpAccount } from '../services/db';
import { parseOtpAuthUri } from '../services/uriParser';

// A placeholder for a Fluent-style toggle switch component
const ToggleSwitch = () => (
  <div className="w-12 h-6 bg-slate-200 dark:bg-slate-700 rounded-full flex items-center p-1 cursor-pointer">
    <div className="w-4 h-4 bg-white rounded-full shadow-md transform transition-transform" />
  </div>
);


export default function SettingsPage() {
  const handleExport = async () => {
    try {
      const accounts = await getAllAccounts();
      const jsonString = JSON.stringify(accounts, null, 2);
      const blob = new Blob([jsonString], { type: 'application/json' });
      const url = URL.createObjectURL(blob);
      const link = document.createElement('a');
      link.href = url;
      const date = new Date().toISOString().slice(0, 10);
      link.download = `totp-backup-${date}.json`;
      document.body.appendChild(link);
      link.click();
      document.body.removeChild(link);
      URL.revokeObjectURL(url);
    } catch (error) {
      console.error('Failed to export data:', error);
      // Here you might want to show a notification to the user
    }
  };

  const handleExportTxt = async () => {
    try {
      const accounts = await getAllAccounts();
      const formatAccount = (acc: TotpAccount) => {
        const name = acc.name.replace(/\s/g, ''); // Remove spaces from name
        return `${name} ${acc.digits} ${acc.secret}`;
      };
      const txtString = accounts.map(formatAccount).join('\n');
      const blob = new Blob([txtString], { type: 'text/plain' });
      const url = URL.createObjectURL(blob);
      const link = document.createElement('a');
      link.href = url;
      const date = new Date().toISOString().slice(0, 10);
      link.download = `totp-backup-${date}.txt`;
      document.body.appendChild(link);
      link.click();
      document.body.removeChild(link);
      URL.revokeObjectURL(url);
    } catch (error) {
      console.error('Failed to export data as TXT:', error);
    }
  };

  const handleExportUriTxt = async () => {
    try {
      const accounts = await getAllAccounts();
      const formatAccountToUri = (acc: TotpAccount) => {
        const totp = new OTPAuth.TOTP({
          issuer: acc.issuer,
          label: acc.name,
          algorithm: acc.algorithm,
          digits: acc.digits,
          period: acc.period,
          secret: acc.secret,
        });
        return totp.toString();
      };
      const uriString = accounts.map(formatAccountToUri).join('\n');
      const blob = new Blob([uriString], { type: 'text/plain' });
      const url = URL.createObjectURL(blob);
      const link = document.createElement('a');
      link.href = url;
      const date = new Date().toISOString().slice(0, 10);
      link.download = `totp-uri-backup-${date}.txt`;
      document.body.appendChild(link);
      link.click();
      document.body.removeChild(link);
      URL.revokeObjectURL(url);
    } catch (error) {
      console.error('Failed to export data as URI TXT:', error);
    }
  };

  const fileInputRef = useRef<HTMLInputElement>(null);

  const parseTxtContent = (content: string): TotpAccount[] => {
    const lines = content.split('\n').filter(line => line.trim() !== '');
    return lines.map(line => {
      const parts = line.split(/\s+/);
      if (parts.length < 3) throw new Error(`Invalid line format: ${line}`);
      
      const name = parts[0];
      const digits = parseInt(parts[1], 10);
      const secret = parts[2];

      if (![6, 7, 8].includes(digits)) throw new Error(`Invalid digits in line: ${line}`);

      // We assume default values for other fields as they are not in the txt format
      return {
        name,
        secret,
        digits: digits as 6 | 7 | 8,
        issuer: '', // Default value
        period: 30, // Default value
        algorithm: 'SHA1', // Default value
        createdAt: new Date(),
      };
    });
  };

  const handleFileSelect = (event: React.ChangeEvent<HTMLInputElement>) => {
    const file = event.target.files?.[0];
    if (!file) return;

    const reader = new FileReader();
    reader.onload = async (e) => {
      try {
        const text = e.target?.result;
        if (typeof text !== 'string') {
          throw new Error('Failed to read file content.');
        }

        let accounts: TotpAccount[];
        if (file.name.endsWith('.json')) {
          const data = JSON.parse(text);
          if (!Array.isArray(data) || !data.every(item => 'name' in item && 'secret' in item)) {
            throw new Error('Invalid JSON file format.');
          }
          accounts = data as TotpAccount[];
        } else if (file.name.endsWith('.txt')) {
          if (text.trim().startsWith('otpauth://')) {
            const lines = text.split('\n').filter(line => line.trim().startsWith('otpauth://'));
            accounts = lines.map(line => {
              const parsed = parseOtpAuthUri(line);
              return {
                ...parsed,
                createdAt: new Date(),
              } as TotpAccount;
            });
          } else {
            accounts = parseTxtContent(text);
          }
        } else {
          throw new Error('Unsupported file type. Please select a .json or .txt file.');
        }

        const importedCount = await importAccounts(accounts);
        alert(`${importedCount} new account(s) imported successfully. ${accounts.length - importedCount} duplicates were skipped.`);
      } catch (error) {
        console.error('Failed to import data:', error);
        alert(`Error importing file: ${error instanceof Error ? error.message : 'Unknown error'}`);
      } finally {
        if (fileInputRef.current) {
          fileInputRef.current.value = '';
        }
      }
    };
    reader.readAsText(file);
  };

  const triggerFileSelect = () => {
    fileInputRef.current?.click();
  };

  const buttonStyles = "rounded-lg bg-indigo-600 px-4 py-2 text-sm font-semibold text-white shadow-fluent-sm hover:bg-indigo-700 transition-colors";

  return (
    <div className="space-y-8">
      <header>
        <h1 className="text-3xl font-bold tracking-tight text-slate-900 dark:text-slate-100">
          Settings
        </h1>
        <p className="mt-1 text-slate-600 dark:text-slate-400">
          Manage application settings and data.
        </p>
      </header>

      <div className="space-y-10">
        {/* Appearance Section */}
        <section>
          <h2 className="text-lg font-semibold text-slate-800 dark:text-slate-200 border-b border-slate-300 dark:border-slate-700 pb-2">
            Appearance
          </h2>
          <div className="mt-4 bg-white dark:bg-slate-900 rounded-xl shadow-fluent-md">
            <div className="p-6 flex items-center justify-between">
              <div>
                <h3 className="font-medium">Dark Mode</h3>
                <p className="text-sm text-slate-500 dark:text-slate-400">Enable or disable dark theme.</p>
              </div>
              <ToggleSwitch />
            </div>
          </div>
        </section>

        {/* Data Management Section */}
        <section>
          <h2 className="text-lg font-semibold text-slate-800 dark:text-slate-200 border-b border-slate-300 dark:border-slate-700 pb-2">
            Data Management
          </h2>
          <div className="mt-4 bg-white dark:bg-slate-900 rounded-xl shadow-fluent-md">
            <div className="p-6 space-y-4">
              <div>
                <h3 className="font-medium">Import Data</h3>
                <p className="text-sm text-slate-500 dark:text-slate-400 mt-1">Import accounts from a previously exported file. Duplicates will be skipped.</p>
              </div>
              <input
                type="file"
                ref={fileInputRef}
                onChange={handleFileSelect}
                className="hidden"
                accept=".json,.txt"
              />
              <button 
                onClick={triggerFileSelect}
                className={buttonStyles}
              >
                Import from File...
              </button>
            </div>
            <hr className="border-slate-200 dark:border-slate-800" />
            <div className="p-6 space-y-4">
              <div>
                <h3 className="font-medium">Export Data</h3>
                <p className="text-sm text-slate-500 dark:text-slate-400 mt-1">Export all your accounts to a JSON file for backup.</p>
              </div>
              <div className="flex items-center space-x-2">
                <button 
                  onClick={handleExport}
                  className={buttonStyles}
                >
                  Export to JSON
                </button>
                <button 
                  onClick={handleExportTxt}
                  className={buttonStyles}
                >
                  Export as Text
                </button>
                <button 
                  onClick={handleExportUriTxt}
                  className={buttonStyles}
                >
                  Export as URI Text
                </button>
              </div>
            </div>
          </div>
        </section>
      </div>
    </div>
  );
}
