import { openDB, type DBSchema, type IDBPDatabase } from 'idb';

export interface TotpAccount {
  id?: number;
  name: string;
  issuer?: string;
  secret: string;
  digits: 6 | 7 | 8;
  period: number;
  algorithm: 'SHA1' | 'SHA256' | 'SHA512';
  createdAt: Date;
}

interface TotpDB extends DBSchema {
  accounts: {
    key: number;
    value: TotpAccount;
    indexes: { 'name': string };
  };
}

const DB_NAME = 'TotpAuthenticatorDB';
const DB_VERSION = 1;

let dbPromise: Promise<IDBPDatabase<TotpDB>> | null = null;

export function getDB(): Promise<IDBPDatabase<TotpDB>> {
  if (!dbPromise) {
    dbPromise = openDB<TotpDB>(DB_NAME, DB_VERSION, {
      upgrade(db) {
        const store = db.createObjectStore('accounts', {
          keyPath: 'id',
          autoIncrement: true,
        });
        store.createIndex('name', 'name', { unique: false });
      },
    });
  }
  return dbPromise;
}

export async function addAccount(account: TotpAccount): Promise<number> {
  const db = await getDB();
  return db.add('accounts', account);
}

export async function getAllAccounts(): Promise<TotpAccount[]> {
  const db = await getDB();
  return db.getAll('accounts');
}

export async function deleteAccount(id: number): Promise<void> {
  const db = await getDB();
  return db.delete('accounts', id);
}

/**
 * Imports accounts, skipping any duplicates based on the secret key.
 * @param accounts The accounts to import.
 * @returns The number of accounts that were newly imported.
 */
export async function importAccounts(accounts: TotpAccount[]): Promise<number> {
  const db = await getDB();
  const existingAccounts = await db.getAll('accounts');
  const existingSecrets = new Set(existingAccounts.map(acc => acc.secret));

  const newAccounts = accounts.filter(acc => !existingSecrets.has(acc.secret));

  if (newAccounts.length === 0) {
    return 0;
  }

  const tx = db.transaction('accounts', 'readwrite');
  await Promise.all(newAccounts.map(account => {
    const { id, ...accountData } = account;
    return tx.store.add(accountData as TotpAccount);
  }));
  await tx.done;

  return newAccounts.length;
}
