import * as OTPAuth from 'otpauth';
import type { TotpAccount } from './db';

/**
 * Parses an otpauth:// URI and converts it into a TotpAccount object.
 *
 * @param uri The otpauth:// URI string.
 * @returns A partial TotpAccount object with the parsed data.
 * @throws An error if the URI is invalid, not a TOTP URI, or cannot be parsed.
 */
export const parseOtpAuthUri = (uri: string): Partial<TotpAccount> => {
  try {
    const otp = OTPAuth.URI.parse(uri);

    if (!(otp instanceof OTPAuth.TOTP)) {
      throw new Error('Only totp URIs are supported.');
    }

    // Type validation and casting
    const algorithm = otp.algorithm as 'SHA1' | 'SHA256' | 'SHA512';
    const digits = otp.digits as 6 | 7 | 8;

    if (!['SHA1', 'SHA256', 'SHA512'].includes(algorithm)) {
      throw new Error(`Unsupported algorithm: ${algorithm}`);
    }
    if (![6, 7, 8].includes(digits)) {
      throw new Error(`Unsupported digits: ${digits}`);
    }

    const account: Partial<TotpAccount> = {
      name: otp.label,
      issuer: otp.issuer,
      secret: otp.secret.base32,
      algorithm: algorithm,
      digits: digits,
      period: otp.period,
    };

    return account;
  } catch (error) {
    console.error('Failed to parse otpauth URI:', error);
    if (error instanceof Error) {
      throw new Error(`Invalid otpauth URI: ${error.message}`);
    }
    throw new Error('An unknown error occurred while parsing the URI.');
  }
};
